@@ -41,10 +41,13 @@ XML::Filter::XML_Directory_2XHTML - SAX2 filter for munging XML::Directory::SAX
  use XML::Directory::SAX;
  use XML::Filter::XML_Directory_2XHTML;
 
- my $file   = IO::File->new(">/htdocs/INC-0.html");
+ my $file   = IO::File->new(">/htdocs/myimages/index.html");
  my $writer = XML::SAX::Writer->new(Output=>$file);
  my $filter = XML::Filter::XML_Directory_2XHTML->new(Handler=>$writer);
 
+ # As Canadian as possible, under the circumstances
+ $filter->set_lang("en-ca");
+
  # Define some images to associate with directory listing.
 
  $filter->set_images({
@@ -82,12 +85,12 @@ XML::Filter::XML_Directory_2XHTML - SAX2 filter for munging XML::Directory::SAX
  # XML::Filter::XML_Directory_Pruner which provides hooks for 
  # restricting the output of XML::Directory::SAX
 
- $filter->exclude(exclude=>["CVS"]);
+ $filter->exclude(ending=>[".html"]);
 
  my $directory = XML::Directory::SAX->new(depth=>0,detail=>2,Handler=>$filter);
 
  $directory->order_by("a");
- $directory->parse_dir($INC[0]);
+ $directory->parse_dir("/htdocs/myimages");
 
 =head1 DESCRIPTION
 
@@ -102,9 +105,9 @@ use Carp;
 use Exporter;
 use File::Basename;
 
-use XML::Filter::XML_Directory_2::Base;
+use XML::Filter::XML_Directory_2::Base '1.4.1';
 
-$XML::Filter::XML_Directory_2XHTML::VERSION   = '1.0';
+$XML::Filter::XML_Directory_2XHTML::VERSION   = '1.2.1';
 @XML::Filter::XML_Directory_2XHTML::ISA       = qw (Exporter XML::Filter::XML_Directory_2::Base);
 @XML::Filter::XML_Directory_2XHTML::EXPORT    = qw();
 @XML::Filter::XML_Directory_2XHTML::EXPORT_OK = qw ();
@@ -215,6 +218,17 @@ Object constructor. Returns an object. Woot!
 
 =cut
 
+=head2 $pkg->set_lang($lang)
+
+Set the language code to be assigned to the <html@xml:lang> and <html@lang> attributes.
+
+=cut
+
+sub set_lang {
+  my $self = shift;
+  $self->{__PACKAGE__.'__lang'} = $_[0];
+}
+
 =head2 $pkg->set_images(\%args)
 
 Define image files to be included with a file or a directory.
@@ -237,7 +251,7 @@ I<string> - the value returned by the I<MIME::Types::mediaType> function for a d
 
 =back
 
-Each key defines it own hash reference whose keys are :
+Each key defines a value which is also a hash reference whose keys are :
 
 =over
 
@@ -267,6 +281,10 @@ String.
 
 =back
 
+Alternately, you may pass a code reference as the key value. If you do, your code reference wil be passed the absolute path of the current file as the first, and only, argument.
+
+Your code reference should return a hash reference whose key/value pairs are the same as those outlined above.
+
 =cut
 
 sub set_images {
@@ -279,8 +297,15 @@ sub set_images {
   }
 
   foreach my $img (keys %$args) {
-    if (ref($args->{$img}) ne "HASH") {
-      carp "Images must be passed as a hash ref of hash references.";
+    my $ref = ref($args->{$img});
+
+    unless ($ref =~ /^(HASH|CODE)$/) {
+      carp "Images must be passed as a hash ref of hash references or code references.";
+      next;
+    }
+
+    if ($ref eq "CODE") {
+      $self->{'__images'}{$img} = $args->{$img};
       next;
     }
 
@@ -476,7 +501,7 @@ sub _stylesheets {
 							  }});
     $self->SUPER::end_element({Name=>"link"});
   }
- 
+
   return 1;
 }
 
@@ -506,7 +531,7 @@ sub _scripts {
 sub _image {
   my $self = shift;
   my $type = shift;
-  my $alt  = shift;
+  my $data = shift;
 
   if (! $type) {
     return 0;
@@ -518,6 +543,17 @@ sub _image {
     return 0;
   }
 
+  if (ref($src) eq "CODE") {
+    $src = &$src($self->build_uri($data).$self->current_location());
+
+    if (ref($src) ne "HASH") { return 0; }
+
+    foreach ("src","height","width") {
+      if (! $src->{$_}) { return 0; }
+    }
+
+  }
+
   $self->SUPER::start_element({Name=>"img",Attributes=>{
 							"{}src"  => {Name=>"src",
 								     Value=>$src->{'src'},
@@ -598,7 +634,14 @@ sub start_document {
 
   $self->SUPER::start_prefix_mapping({Prefix => "",
 				     NamespaceURI => "http://www.w3.org/1999/xhtml"});
-  $self->SUPER::start_element({Name=>DTD_HTML_ROOT,__PACKAGE__->attributes(lang=>"en-ca","xml:lang"=>"en-ca")});
+
+  my %attrs = ();
+
+  if (my $lang = $self->{__PACKAGE__.'__lang'}) {
+    %attrs = __PACKAGE__->attributes(lang=>$lang,"xml:lang"=>$lang);
+  }
+
+  $self->SUPER::start_element({Name=>DTD_HTML_ROOT,%attrs});
   $self->SUPER::end_prefix_mapping({Prefix=>""});
 
   return 1;
@@ -627,7 +670,9 @@ sub start_element {
   my $self = shift;
   my $data = shift;
 
-  $self->on_enter_start_element($data) || return 0;
+  if (! $self->on_enter_start_element($data)) {
+    return 0;
+  }
 
   if ($data->{Name} =~ /^(file|directory)$/) {
     my $name = lc $1;
@@ -669,7 +714,7 @@ sub end_element {
 
     #
 
-    my $title = $self->build_uri();
+    my $title = $self->current_location() || &basename($self->build_uri($data));
 
     if (my $c = $self->get_callback("title")) {
       $title = &$c();
@@ -717,11 +762,11 @@ sub characters {
 
 =head1 VERSION
 
-1.0
+1.2.1
 
 =head1 DATE
 
-July 02, 2002
+July 07, 2002
 
 =head1 AUTHOR
 
@@ -739,10 +784,6 @@ Add hooks to set <meta> tags
 
 Add hooks to set <link> tags
 
-=item *
-
-Add hooks to set <html@lang> attribute
-
 =back
 
 =head1 SEE ALSO
